
import logging

import pygame

log = logging.getLogger('keyboard')


class Binding(object):
    def __init__(self, key, func):
        self._key = key
        self._func = func
        self._state = None


class KeyboardEvent(object):
    def __init__(self, key, event_type, binding=None):
        self.key = key
        self.unicode = ''
        self.binding = binding
        self.time = pygame.time.get_ticks()
        self.event_type = event_type

    def __repr__(self):
        return '<KeyboardEvent(key:{} @ time:{})>'.format(self.key, self.time)


class Keymap(object):
    """
    Keymap
    """
    def __init__(self):
        self._bindings = {}
        self._pressed = {}

    def add_binding(self, key, func):
        if key in self._bindings:
            error_msg = '{} is already bound to {}'.format(
                key, self._bindings[key])
            raise ValueError(error_msg)
        self._bindings[key] = func
        log.debug('Bound {} to {}'.format(key, func))

    def _key_pressed(self, event):
        self._pressed[event.key] = event
        if event.binding is not None:
            event.binding(event)
        log.debug('{} has been pressed'.format(self._pressed[event.key]))
        return event

    def _key_released(self, event):
        keyboard_event = None
        try:
            keyboard_event = self._pressed[event.key]
        except KeyError:
            log.error('{} was not in the pressed list'.format(event.key))

        if event.binding is not None:
            event.binding(event)

        if keyboard_event:
            del self._pressed[event.key]
            held_for = pygame.time.get_ticks() - keyboard_event.time
            log.debug('{} has been released. held for: {}ms'.format(
                keyboard_event, held_for))
            return keyboard_event
        return None

    def push_event(self, event):
        binding = None

        if event.key in self._bindings:
            binding = self._bindings[event.key]

        kb_event = KeyboardEvent(event.key, event.type, binding)

        if event.type is pygame.KEYDOWN:
            kb_event.unicode = event.unicode
            return self._key_pressed(kb_event)

        elif event.type is pygame.KEYUP:
            return self._key_released(kb_event)

    def fake_press(self, key, mod=0):
        """
        KMOD_NONE, KMOD_LSHIFT, KMOD_RSHIFT, KMOD_SHIFT, KMOD_CAPS,
        KMOD_LCTRL, KMOD_RCTRL, KMOD_CTRL, KMOD_LALT, KMOD_RALT,
        KMOD_ALT, KMOD_LMETA, KMOD_RMETA, KMOD_META, KMOD_NUM, KMOD_MODE
        """
        log.error('TODO')
        self.push_event(event)

    def fake_release(self, key):
        log.error('TODO')

    def update(self):
        pass
