import textwrap
import json
import logging

import pygame


class GameSprite(pygame.sprite.DirtySprite):
    def __init__(self, window):
        super(GameSprite, self).__init__()
        self._window = window


class SpriteBatch(object):
    def __init__(self):
        self._sprites = []

    def add(self, sprite):
        self._sprites.append(sprite)

    def remove(self, sprite):
        self._sprites.remove(sprite)

    def draw(self, dest):
        update_rects = []
        for l in self._sprites:
            if l.dirty == 1:
                # print 'blitting', l, l.rect
                dest.blit(l.image, l.rect)
                l.dirty = 0
                update_rects += [l.rect]
        return update_rects


class Layer(object):
    def __init__(self, window):
        self._batch = SpriteBatch()
        self._window = window


class TiledSprite(GameSprite):
    def __init__(self, window, filename):
        super(TiledSprite, self).__init__(window)
        tiled_data = json.loads(open(filename).read())
        self._properties = tiled_data['properties']
        self._layers = []
        for l in tiled_data['layers']:
            self._layers.append(l)
        self.image = pygame.surface.Surface((1280, 720))
        self.rect = (0, 0, 1280, 720)
        self._render()

    def _render(self):
        for l in self._layers:
            n = 0
            color = l['properties']['color']
            for d in l['data']:
                x, y = n % 80, n / 80
                if d != 0:
                    sheet = self._window.colors[color]
                    rect = sheet.draw_tile(self.image, x, y, d-1)
                n += 1
        self.dirty = 1


class TiledLayer(Layer):
    def __init__(self, window, filename):
        super(TiledLayer, self).__init__(window)
        self._batch.add(TiledSprite(window, filename))


class TextItem(GameSprite):
    def __init__(self, x, y, text, color, window):
        super(TextItem, self).__init__(window)
        self._x = x
        self._y = y
        self._text = text
        self._color = color

        self.image = None
        self.rect = None

        self._render()

    def set_text(self, new_text):
        self._text = new_text
        self.dirty = 1
        self._render()

    def _render(self):
        tl = len(self._text)
        new_surf = pygame.surface.Surface((tl*16, 16), pygame.SRCALPHA)
        n = 0
        sheet = self._window.colors[self._color]
        for c in self._text:
            sheet.draw_tile(new_surf, n, 0, ord(c))
            n += 1
        self.image = new_surf
        rect = new_surf.get_rect()
        rect.x = self._x*16
        rect.y = self._y*16
        self.rect = rect


class TextLayer(Layer):
    def __init__(self, window):
        super(TextLayer, self).__init__(window)

    def add_string(self, x, y, text, color='default'):
        ti = TextItem(x, y, text, color, self._window)
        self._batch.add(ti)
        return ti

    def add_text(self, x, y, text):
        text = text.split('\n')
        n = 0
        for l in text:
            for sl in textwrap.wrap(l, 78):
                self.add_string(x, y+n, sl)
                n += 1
            if l == '':
                n += 1


class MenuSprite(GameSprite):
    def __init__(self, window, x, y, w, h):
        super(MenuSprite, self).__init__(window)
        self._x = x
        self._y = y
        self._w = w
        self._h = h
        self.image = None
        self.rect = (x*16, y*16, w*16, h*16)
        self._menu_items = []
        self._render()
        self.selected = 0

    def _render(self):
        self.image = pygame.surface.Surface((self._w*16, self._h*16))
        sheet = self._window.colors['menu']
        sel_sheet = self._window.colors['menu_selected']
        sheet.draw_window(self.image, self._w, self._h)
        x, y = 2, 1
        for mi in range(0, len(self._menu_items)):
            label, func = self._menu_items[mi]
            n = 0
            for c in label:
                if mi == self.selected:
                    sel_sheet.draw_tile(self.image, x+n, y+0, ord(c))
                else:
                    sheet.draw_tile(self.image, x+n, y+0, ord(c))
                n += 1
            y += 1
        self.dirty = 1

    def add_menu_item(self, label, func):
        self._menu_items.append((label, func))
        self._render()

    def next(self):
        if self.selected < len(self._menu_items)-1:
            self.selected += 1
            self._render()

    def prev(self):
        if self.selected > 0:
            self.selected -= 1
            self._render()

    def activate(self):
        self._menu_items[self.selected][1]()


class MenuLayer(Layer):
    def __init__(self, window, x, y, w, h):
        super(MenuLayer, self).__init__(window)
        self._sprite = MenuSprite(window, x, y, w, h)
        self._batch.add(self._sprite)

    def add_menu_item(self, label, func):
        self._sprite.add_menu_item(label, func)

    def next(self):
        self._sprite.next()

    def prev(self):
        self._sprite.prev()

    def activate(self):
        self._sprite.activate()

    def remove(self):
        self._batch.remove(self._sprite)


class PingPong(GameSprite):
    def __init__(self, window, x, y, w, hw, speed):
        super(PingPong, self).__init__(window)
        self.image = None
        self.rect = (x*16, y*16, w*16, 16)
        self._x = x
        self._y = y
        self._w = w
        self._hw = hw
        self._speed = speed
        self._pos = 0
        self._direction = 1
        self._render()

    def _render(self):
        self.image = pygame.surface.Surface((16*self._w, 16))
        sheet = self._window.colors['pp_back']
        hit_sheet = self._window.colors['pp_hz']
        hi_sheet = self._window.colors['pp_hi']
        hs = (self._w/2) - (self._hw/2)
        he = (self._w/2) + (self._hw/2)
        for n in range(0, self._w):
            if n in range(hs, he):
                hit_sheet.draw_tile(self.image, n, 0, 178)
            else:
                sheet.draw_tile(self.image, n, 0, 177)
        hi_sheet.draw_tile(self.image, self._pos, 0, 33)
        self.dirty = 1

    def update(self):
        # print '_update()', self._pos
        if self._pos < self._w and self._pos >= 0:
            self._pos += self._direction
            if self._pos == 0:
                self._direction = 1
            if self._pos == self._w-1:
                self._direction = -1
        else:
            pass
        self._render()

    def check_hit(self):
        hs = (self._w/2) - (self._hw/2)
        he = (self._w/2) + (self._hw/2)
        if self._pos in range(hs, he):
            return True
        else:
            return False


class PingPongLayer(Layer):
    def __init__(self, window, x, y, w, hw, speed):
        super(PingPongLayer, self).__init__(window)
        pp = PingPong(window, x, y, w, hw, speed)
        self._batch.add(pp)
        self._sprite = pp
        self._timer = self._window.scheduler.new_cycle_timer(
            speed/w, self._sprite.update)

    def stop(self):
        # print 'ppl.stop()'
        self._timer.kill()

    def check_hit(self):
        return self._sprite.check_hit()


class HPDisplay(GameSprite):
    def __init__(self, window, x, y):
        super(HPDisplay, self).__init__(window)
        self.image = None
        self.rect = (x*16, y*16, 20*16, 16)
        self._render()

    def _render(self):
        self.image = pygame.surface.Surface((20*16, 16))
        self.image.fill((0, 0, 255))
        self.dirty = True


class BattleLayer(Layer):
    def __init__(self, window):
        super(BattleLayer, self).__init__(window)
        self.player_hp = HPDisplay(window, 10, 10)
        self._batch.add(self.player_hp)


class Scene(object):
    def __init__(self, window):
        self._window = window
        self._layers = []

    def add_tiled_layer(self, filename):
        new_tiled_layer = TiledLayer(self._window, filename)
        self._layers.append(new_tiled_layer)
        return new_tiled_layer

    def add_text_layer(self):
        new_text_layer = TextLayer(self._window)
        self._layers.append(new_text_layer)
        return new_text_layer

    def add_menu_layer(self, x, y, w, h):
        new_menu_layer = MenuLayer(self._window, x, y, w, h)
        self._layers.append(new_menu_layer)
        return new_menu_layer

    def add_ping_pong_layer(self, x, y, w, hw, speed):
        new_pp_layer = PingPongLayer(self._window, x, y, w, hw, speed)
        self._layers.append(new_pp_layer)
        return new_pp_layer

    def add_battle_layer(self):
        new_battle_layer = BattleLayer(self._window)
        self._layers.append(new_battle_layer)
        return new_battle_layer

    def draw(self, window):
        update_rects = []
        for l in self._layers:
            update_rects += l._batch.draw(self._window.screen)
        return update_rects

    def redraw_all(self):
        for l in self._layers:
            for b in l._batch._sprites:
                b.dirty = 1

    def remove_layer(self, layer):
        self._layers.remove(layer)
        self.redraw_all()
