import logging

import pygame


log = logging.getLogger('Scheduler')


class Timer(object):
    def __init__(self):
        self._alive = True
        self._update_count = 0
        self._update_pool = 0

    def kill(self):
        log.info('Killed: {}'.format(self))
        self._alive = False

    def is_alive(self):
        return self._alive


class UpdatesPerSecTimer(Timer):
    def __init__(self, update_target):
        super(UpdatesPerSecTimer, self).__init__()
        self._update_target = update_target

    def update(self, dt):
        self._update_pool += dt

    def should_update(self):
        if self._update_pool >= 1000/self._update_target:
            self._update_pool -= 1000/self._update_target
            self._update_count += 1
            return True
        else:
            return False


class CycleTimer(Timer):
    def __init__(self, delay, callback, one_shot=False):
        super(CycleTimer, self).__init__()
        self._delay = delay
        self._callback = callback
        self._one_shot = one_shot

    def update(self, dt):
        self._update_pool += dt
        if self._update_pool >= self._delay:
            self._callback()
            self._update_pool -= self._delay
            if self._one_shot:
                self.kill()


class Scheduler(object):
    def __init__(self):
        self.current_time = 0
        self._last_time = 0
        self._timers = []

    def update(self, current_time, dt):
        self.current_time = current_time
        self._last_time = current_time
        timers_to_remove = []
        for timer in self._timers:
            timer.update(dt)
            if timer.is_alive() is False:
                timers_to_remove.append(timer)
        for to_remove in timers_to_remove:
            self._timers.remove(to_remove)

    def new_updates_timer(self, target):
        new_timer = UpdatesPerSecTimer(target)
        self._timers.append(new_timer)
        return new_timer

    def new_cycle_timer(self, delay, func):
        new_timer = CycleTimer(delay, func)
        self._timers.append(new_timer)
        return new_timer

    def new_oneshot_timer(self, delay, func):
        new_timer = CycleTimer(delay, func, True)
        self._timers.append(new_timer)
        return new_timer


if __name__ == '__main__':
    class CallbackMethods(object):
        def __init__(self):
            self.cycles = 0

        def foo_cycle(self):
            self.cycles += 1
            print 'foo_cycle', self.cycles

        def foo_shot(self):
            print 'foo_shot'

    cbm = CallbackMethods()
    sched = Scheduler()
    frame_timer = sched.new_updates_timer(60)
    cycle = sched.new_cycle_timer(100, cbm.foo_cycle)
    cycle = sched.new_oneshot_timer(7000, cbm.foo_shot)
    n = 0
    last_time = 0
    while True:
        current_time = sched.current_time
        if current_time > last_time + 1000:
            last_time = current_time
            n += 1
            print n, '\n'

        sched.update()
        if frame_timer.should_update():
            pass

        pygame.time.wait(1)
