import logging
import time

import pygame
from pygame.locals import *


from colors import STANDARD
from ascii import AsciiSheet
from schedule import Scheduler


class Window(object):
    def __init__(self, width, height, flags=0):
        self.log = logging.getLogger('Window')
        self.log.info('++ Creating new window object ({},{})'.format(width,
                                                                     height))
        self.width = width
        self.height = height
        pygame.init()
        self.screen = pygame.display.set_mode((width, height), flags)
        self.state_stack = []
        self.running = True
        self.last_state = None
        self.last_time = pygame.time.get_ticks()
        # Stuff for FPS calc
        self.last_fps_time = self.last_time
        self.frame_count = 0
        self.update_count = 0
        self.fps = 0
        self.ups = 0
        self.colors = {}
        for k, v in STANDARD.items():
            self.colors[k] = AsciiSheet('assets/fonts/standard_ascii.png', 16, v[0], background=v[1])
        self.scheduler = Scheduler()
        self.gfx_timer = self.scheduler.new_updates_timer(60)
        self.update_timer = self.scheduler.new_updates_timer(100)

    def push_state(self, state_cls, *args, **kwargs):
        state = state_cls(self, *args, **kwargs)
        ss_len = len(self.state_stack)
        if ss_len > 0:
            top_state = self.state_stack[ss_len-1]
            top_state.on_state_lose_focus()
        self.log.info('>> Pushing: {}'.format(state.name))
        self.state_stack.append(state)

    def pop_state(self, state=None):
        # Check to selectively remove a state from the stack rather than pop it
        if state is not None:
            self.state_stack.remove(state)
            self.log.info('<< Popped: {}'.format(state.name))
            return state
        # Call the change in focus
        popped_state = self.state_stack.pop()
        popped_state.on_state_lose_focus()
        self.log.info('<< Popped: {}'.format(popped_state.name))
        return popped_state

    def run(self):
        while self.running:
            # Check if state stack is empty
            if len(self.state_stack) == 0:
                self.running = False
                break

            ss_len = len(self.state_stack)
            top_state = self.state_stack[ss_len-1]

            # Check to see if the top state has changed and call focus
            if top_state is not self.last_state:
                self.last_state = top_state
                top_state.on_state_gain_focus()

            # Send events to or state
            for event in pygame.event.get():
                if event.type == KEYDOWN:
                    top_state.on_keydown(event)
                elif event.type == KEYUP:
                    top_state.on_keyup(event)
                elif event.type == QUIT:
                    top_state.on_quit()
                elif event.type == ACTIVEEVENT:
                    if event.gain == 1:
                        top_state.on_window_gain_focus()
                    else:
                        top_state.on_window_lose_focus()
                elif event.type == MOUSEMOTION:
                    top_state.on_mouse_motion(event)
                elif event.type == MOUSEBUTTONDOWN:
                    top_state.on_mouse_button_down(event)
                elif event.type == MOUSEBUTTONUP:
                    top_state.on_mouse_button_up(event)
                elif event.type == VIDEORESIZE:
                    top_state.on_window_resize(event)
                else:
                    self.log.info('Unhandled event: {}'.format(event))

            # calc delta time and call update
            current_time = pygame.time.get_ticks()
            delta_time = (current_time - self.last_time)
            self.scheduler.update(current_time, delta_time)

            any_update = False
            if self.update_timer.should_update():
                top_state.on_update(delta_time)
                self.update_count += 1
                any_update = True
            self.last_time = current_time

            # Call draw
            if self.gfx_timer.should_update():
                self.frame_count += 1
                top_state.on_draw()
                any_update = True

            if current_time > (self.last_fps_time + 1000):
                self.fps = self.frame_count
                self.ups = self.update_count
                self.last_fps_time = current_time
                self.frame_count = 0
                self.update_count = 0

            if any_update is False:
                pygame.time.wait(10)

    def new_sprite(self):
        pass
